#
# Author:: Kevin Moser (<kevin.moser@nordstrom.com>)
#
# Copyright:: 2012, Nordstrom, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
require 'socket'

class MetricCollect
  class Backend
    class Statsd 
      def initialize(name)
        @params = Hash.new
        @params[:name] = name
        @params[:value] = nil
        @params[:key] = "metric_collect"
        @params[:type] = :gauge
      end

      def method_missing(method_symbol, *args, &block)
        @params[method_symbol]
      end

      attr_accessor :params

      def process_metric(config)
        @config = config.params
        name = @params[:name]
        value = @params[:value]
        key = @params[:key]
        type = @params[:type]

        case type
        when :counter
          counters("#{key}.#{name}", value)
        when :timer
          timers("#{key}.#{name}", value)
        when :gauge
          gauges("#{key}.#{name}", value)
        when :set
          sets("#{key}.#{name}", value)
        else
          MetricCollect::Log.fatal("#{type.to_s} is not a valid statsd type!")
        end
      end

      private
      def timers(stats, time, sample_rate=1)
        update_stats(stats, time, sample_rate, 'ms')
      end

      def counters(stats, value, sample_rate=1)
        update_stats(stats, value, sample_rate)
      end

      def gauges(stats, value, sample_rate=1)
        update_stats(stats, value, sample_rate, 'g')
      end

      def sets(stats, value, sample_rate=1)
        update_stats(stats, value, sample_rate, 's')
      end

      def update_stats(stats, delta=1, sample_rate=1, metric='c')
        stats = [stats].flatten

        data = {}
        stats.each do |stat|
          data[stat] = "%s|%s" % [delta, metric]
        end

        send(data, sample_rate)
      end

      def send(data, sample_rate=1)
        begin
          host = @config[:statsd_server]
          port = @config[:statsd_port]

          sampled_data = {}
          if sample_rate < 1
            if rand <= sample_rate
              data.each_pair do |stat, val|
                sampled_data[stat] = "%s|@%s" % [val, sample_rate]
              end
            end
          else
            sampled_data = data
          end

          udp = UDPSocket.new
          sampled_data.each_pair do |stat, val|
            send_data = "%s:%s" % [stat, val]
            udp.send send_data, 0, host, port
          end
        rescue => e
          puts e.message
        end
      end
    end
  end
end
